package com.webchat.pay.service;


import com.webchat.common.bean.APIResponseBean;
import com.webchat.common.enums.RedisKeyEnum;
import com.webchat.common.service.RedisService;
import com.webchat.common.util.AkSkGenerator;
import com.webchat.common.util.IDGenerateUtil;
import com.webchat.common.util.SignUtil;
import com.webchat.domain.vo.dto.payment.AppAkSkDTO;
import com.webchat.pay.config.exception.PaymentAccessAuthException;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

@Service
public class PaymentApiService {

    @Autowired
    private PaymentAppService appService;

    @Autowired
    private RedisService redisService;


    /**
     * 交易凭证获取签名的有效期：5分钟
     */
    private static final Long SIGNATURE_EXPIRE_TIME = 5 * 60 * 1000L;

    /**
     * 生成交易凭证
     *
     * @param appId
     * @param accessKey
     * @param timestamp
     * @param signature
     * @param logId
     * @return
     */
    public String token(Long appId,
                        String accessKey,
                        String secretKey,
                        Long timestamp,
                        String signature,
                        String logId) {


        /**
         * 这里三重校验
         * ------------------------
         * 1. 校验访问凭证
         * 2. 校验校验有效
         * 3. 校验签名
         */
        // 1. 校验访问凭证
        AppAkSkDTO akSkDTO = appService.getAppAkSk(appId);
        if (akSkDTO == null) {
            throw new PaymentAccessAuthException("应用不存在");
        }
        boolean checkAk = ObjectUtils.equals(akSkDTO.getAccessKey(), accessKey);
        boolean checkSk = AkSkGenerator.checkKey(secretKey, akSkDTO.getSecretHashKey());
        if (!(checkAk && checkSk)) {
            throw new PaymentAccessAuthException("应用访问凭证校验失败");
        }
        // 2. 校验校验有效
        Long diffTime = System.currentTimeMillis() - timestamp;
        if (diffTime > SIGNATURE_EXPIRE_TIME) {
            throw new PaymentAccessAuthException("签名已过期");
        }
        // 3. 校验签名
        String validSignature = SignUtil.generateSignature(secretKey,
                                                           String.valueOf(appId),
                                                           accessKey,
                                                           String.valueOf(timestamp));
        if (!ObjectUtils.equals(signature, validSignature)) {
            throw new PaymentAccessAuthException("签名校验失败");
        }

        /**
         * 生成并返回接入方access-token（交易凭证，用于后续的交易校验）
         */

        return this.generateAccessToken(appId);
    }

    /**
     * 获取交易凭证背后的应用信息
     *
     * @param accessToken
     * @return
     */
    public Long getAppIdFromAccessToken(String accessToken) {
        String accessTokenCacheKey = this.accessTokenCacheKey(accessToken);
        String cache = redisService.get(accessTokenCacheKey);
        if (StringUtils.isNotBlank(cache)) {
            return Long.valueOf(cache);
        }
        return null;
    }

    private String generateAccessToken(Long appId) {
        /**
         * 生成交易凭证
         */
        String accessToken = IDGenerateUtil.uuid();
        String accessTokenCacheKey = this.accessTokenCacheKey(accessToken);
        redisService.set(accessTokenCacheKey, String.valueOf(appId),
                RedisKeyEnum.PAYMENT_APP_ACCESS_TOKEN_CACHE.getExpireTime());
        return accessToken;
    }

    private String accessTokenCacheKey(String accessToken) {

        return RedisKeyEnum.PAYMENT_APP_ACCESS_TOKEN_CACHE.getKey(accessToken);
    }


    /**
     * 校验支付交易接口请求token的有效性
     *
     * @param accessToken
     * @return
     */
    public boolean validateAccessToken(String accessToken) {

        /**
         * token 由支付平台生成且未失效 ===> accToken是有效的
         */
        String cacheKey = this.accessTokenCacheKey(accessToken);
        return redisService.exists(cacheKey);
    }

}
